/* =================================================================
 * Artekit AK-MAG3110 Breakout Board
 * 
 * This is a simple sample sketch that reads raw data from the
 * MAG3110 sensor. It's uses I2C and the interrupt pin.
 *
 * I2C Connection:
 *    SDA on pin A4
 *    SCL on pin A5
 * 
 * Interrupt:
 *    INT on pin 2
 * ================================================================= */
#include <Wire.h>

#define MAG3110_ADDR  0x0E

/* This function reads a byte for the given register */
static byte ReadRegister(byte reg)
{
  Wire.beginTransmission(MAG3110_ADDR);
  Wire.write(reg);
  /* By setting false here we tell the Wire library to
   * repeat the start condition. May not work on older
   * Arduino libraries.
   */
  Wire.endTransmission(false);
  Wire.requestFrom(MAG3110_ADDR, 1);
  return Wire.read();
}

/* This function writes a byte to the given register */

static void WriteRegister(byte reg, byte value)
{
  Wire.beginTransmission(MAG3110_ADDR);
  Wire.write(reg);
  Wire.write(value);
  Wire.endTransmission(true);
}

static void ReadData(byte start, byte* data, int len)
{
  int offs = 0;
  
  Wire.beginTransmission(MAG3110_ADDR);
  Wire.write(start);
  /* By setting false here we tell the Wire library to
   * repeat the start condition. May not work on older
   * Arduino libraries.
   */
  Wire.endTransmission(false);
  Wire.requestFrom(MAG3110_ADDR, len);
  while (len)
  {
     data[offs++] = Wire.read();
     len--;
  }
}

void config(void)
{
  /* Enable auto-resets */
  WriteRegister(0x11, 0xA0);

  /* Set active mode */
  WriteRegister(0x10, 0x01);  
}

void read_and_print(void)
{
  byte data[6];
  int x, y, z;
  
  ReadData(0x01, data, 6);
    
  x = (data[0]<<8) + data[1];
  y = (data[2]<<8) + data[3];
  z = (data[4]<<8) + data[5];

  Serial.print("X = ");  
  Serial.print(x * 0.1f); 
    
  Serial.print(" Y = ");
  Serial.print(y * 0.1f);
   
  Serial.print(" Z = ");
  Serial.println(z * 0.1f);
}

void setup()
{
  Wire.begin();
  Serial.begin(57600);
  config();
  
  pinMode(2, INPUT);
}

void loop()
{
  if (digitalRead(2))
  {
    read_and_print();
  }
}

